//
// XMLEnumerateInstanceNames.java
//
// The contents of this file are subject to the SNIA Public License Version 1.0
// (the "License"); you may not use this file except in compliance with the
// License. You may obtain a copy of the License at
//
//    http://www.snia.org/resources/openSource.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
// License for the specific language governing rights and limitations
// under the License.
//
// The Original Code is XMLEnumerateInstanceNames.java
// 
// The Initial Developer of the Original Code is David Simons.
//
// Contributor(s):
//
// This source uses the Solaris WBEM APIs version 2.2. 
// Synchronization will be maintained with the Solaris WBEM APIS,
// which are being standardized through the Java Community Process.
//
//
// $Id: XMLEnumerateInstanceNames.java,v 1.1 2003/07/17 18:32:55 kumpf Exp $
//
// $Log: XMLEnumerateInstanceNames.java,v $
// Revision 1.1  2003/07/17 18:32:55  kumpf
// HP-BAPU: PEP-64 CIM Client and CIM Listener Java Interface
//
// Revision 1.1.1.1  2001/10/31 18:17:40  karl
// import of SNIA 0_7
//
// Revision 1.7  2000/11/06 19:25:07  davidsi
// - Reconciled error ids.
//
// Revision 1.6  2000/08/16 05:59:59  shawn
// merge P_TOMCAT31 branch
//
// Revision 1.5.8.1  2000/08/10 00:03:36  shawn
// head merge
//
// Revision 1.5  2000/05/11 10:11:24  davidsi
// - Fixed tabs.
//
// Revision 1.4  2000/05/06 17:20:03  davidsi
// - Brought in line with SUN WBEM APIs 2.2
//
// Revision 1.3  2000/04/09 14:39:49  davidsi
// - Fixed extraction and generation of classnames.
//
// Revision 1.2  2000/02/13 15:10:05  davidsi
// - Added text to cover re-use of Solaris WBEM APIs.
//
// Revision 1.1.1.1  2000/02/06 16:41:02  davidsi
// Initial version.
//
//
//
package org.snia.wbemcmd.xml;

import java.util.Enumeration;
import java.util.Vector;
import java.net.MalformedURLException;
import org.snia.wbem.cim.CIMException;
import org.snia.wbem.cim.CIMObjectPath;
import org.snia.wbem.cim.CIMClass;
import org.snia.wbem.cim.CIMInstance;
import org.snia.wbemcmd.utils.SimpleEnum;
import org.snia.wbem.client.CIMOMHandle;

public class XMLEnumerateInstanceNames extends XMLClass {
    static String commandName="EnumerateInstanceNames";
    
    
    public void execute(HTTPWriter pw,XMLNode node,CIMObjectPath path,CIMOMHandle hdl) 
        throws CIMException,MalformedURLException {
    String className;
    Enumeration enum;
    
    className=node.extractParameterValueAttr(XMLP_CLASSNAME,XMLNode.XML_ELEMENT_CLASSNAME,
                           paramName);
    if (className==null) {
        throw new CIMException(CIMException.CIM_ERR_NOT_FOUND,"Class was null");
    }
    
    //
    // Get class
    //
    path.setObjectName(className);
    CIMClass cimClass=hdl.getClass(path,false);

    //
    // Get Instance
    //
    path.setObjectName(className);
    // 
    // Note that while the SUN API allows deep the
    // XML encodings don't so we simply assume deep
    //
    enum=hdl.enumInstances(path,true,false);

    while (enum.hasMoreElements()) {
        CIMInstance ci=(CIMInstance)(enum.nextElement());
        CIMObjectPath cop=new CIMObjectPath(ci.getClassName(),ci.getKeyValuePairs());
        pw.println(cop.toXml());
    }
    
    return;
    }


    //
    // Note that while the SUN API allows deep the
    // XML encodings don't so we simply assume deep
    //

    //
    public static Enumeration enumInstances(HTTPOutput output,
            CIMObjectPath path) throws CIMException {
        return(enumerateInstanceNames(output, path));
    }

    public static Enumeration enumerateInstanceNames(HTTPOutput output,
        CIMObjectPath path) throws CIMException {
    XMLNode result;
    XMLAttribute attr;
    String className=path.getObjectName();
    SimpleEnum se=new SimpleEnum();
    Vector params=new Vector();
    
    output.setMethod(commandName);
    

    if (className!=null)
        params.addElement(new Param(XMLP_CLASSNAME,Param.VALUESET,
            "<CLASSNAME NAME=\""+className+"\"/>"));
    else 
        throw new CIMException(CIMException.CIM_ERR_NOT_FOUND,"Class was null");
    
    result=intrinsicMethod(output,path,commandName,params);
    
    if (result==null) {
        return(se);
    }
    
    //
    // Now process instance values
    //
    
    for(;result!=null;result=result.getNext()) {
        result=result.findElement(XMLNode.XML_ELEMENT_INSTANCENAME);
        if (result==null)
        return(se);
        CIMObjectPath newpath=new CIMObjectPath();
        newpath.setNameSpace(path.getNameSpace());
        getInstanceName(result,newpath);
        se.addElement(newpath);
    }
    return(se);
    }
}
