//
// XMLGetClass.java
//
// The contents of this file are subject to the SNIA Public License Version 1.0
// (the "License"); you may not use this file except in compliance with the
// License. You may obtain a copy of the License at
//
//    http://www.snia.org/resources/openSource.html
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
// License for the specific language governing rights and limitations
// under the License.
//
// The Original Code is XMLGetClass.java
// 
// The Initial Developer of the Original Code is David Simons.
//
// Contributor(s):
//
//
// This source uses the Solaris WBEM APIs version 2.2. 
// Synchronization will be maintained with the Solaris WBEM APIS,
// which are being standardized through the Java Community Process.
//
//
// $Id: XMLGetClass.java,v 1.1 2003/07/17 18:32:55 kumpf Exp $
//
// $Log: XMLGetClass.java,v $
// Revision 1.1  2003/07/17 18:32:55  kumpf
// HP-BAPU: PEP-64 CIM Client and CIM Listener Java Interface
//
// Revision 1.1.1.1  2001/10/31 18:17:44  karl
// import of SNIA 0_7
//
// Revision 1.8  2000/11/12 14:50:26  davidsi
// - Added support for <VALUE.OBJECT>.
//
// Revision 1.7  2000/11/06 19:12:12  davidsi
// - Added support for VALUE_NAMEDOBJECT tags
//
// Revision 1.6  2000/08/16 05:59:59  shawn
// merge P_TOMCAT31 branch
//
// Revision 1.5.8.1  2000/08/10 00:03:36  shawn
// head merge
//
// Revision 1.5  2000/05/11 10:11:24  davidsi
// - Fixed tabs.
//
// Revision 1.4  2000/05/06 17:20:03  davidsi
// - Brought in line with SUN WBEM APIs 2.2
//
// Revision 1.3  2000/04/09 14:40:47  davidsi
// - Fixed generation and extraction of classnames.
// - Removed VALUE.NAMEDOBJECT elements.
//
// Revision 1.2  2000/02/13 15:10:06  davidsi
// - Added text to cover re-use of Solaris WBEM APIs.
//
// Revision 1.1.1.1  2000/02/06 16:41:02  davidsi
// Initial version.
//
//
//
package org.snia.wbemcmd.xml;

import java.util.Enumeration;
import java.util.Vector;
import java.net.MalformedURLException;
import org.snia.wbem.cim.CIMException;
import org.snia.wbem.cim.CIMObjectPath;
import org.snia.wbem.cim.CIMClass;
import org.snia.wbem.client.CIMOMHandle;


public class XMLGetClass extends XMLClass 
{
    static String commandName="GetClass";
       
    public void execute(HTTPWriter pw, XMLNode node, CIMObjectPath path,
        CIMOMHandle hdl) throws CIMException, MalformedURLException 
    {
        String className=null;
        String[] propertyList;
        boolean localOnly;
        boolean includeQualifiers;
        boolean includeClassOrigin;
        CIMClass cimClass;
        
        className=node.extractParameterValueAttr(
            XMLP_CLASSNAME,XMLNode.XML_ELEMENT_CLASSNAME, paramName);

        if (className==null)
            throw new CIMException(CIMException.CIM_ERR_INVALID_CLASS);

        propertyList=node.extractParameterStringArray(XMLP_PROPERTYLIST);
        localOnly=node.extractParameterValue(XMLP_LOCAL,true);
        includeQualifiers=node.extractParameterValue(XMLP_QUAL,true);
        includeClassOrigin=node.extractParameterValue(XMLP_ORIGIN,false);

        path.setObjectName(className);

        //cimClass=hdl.getClass(path, localOnly); 
        cimClass=hdl.getClass(path, localOnly, includeQualifiers, 
            includeClassOrigin, propertyList);
        if (cimClass==null) 
        {
            throw new CIMException(CIMException.CIM_ERR_NOT_FOUND,
                "Path=" + path);
        }   

        pw.println(cimClass.toXml(localOnly, includeQualifiers, 
            includeClassOrigin, propertyList));

        return;
    }
    
    
    public static CIMClass getClass(HTTPOutput output,
        CIMObjectPath path,
        boolean localOnly) throws CIMException 
    {
        return(XMLGetClass.getClass(output, path, localOnly, true, false, null));
    }

    public static CIMClass getClass(HTTPOutput output,
        CIMObjectPath path,
        boolean localOnly,
        boolean includeQualifiers,
        boolean includeClassOrigin,
        String propertyList[]) throws CIMException
    {
        XMLNode result;
        Vector params=new Vector();
        XMLAttribute attr;
        String className=path.getObjectName();
        String extra="";

        output.setMethod(commandName);
        if (className!=null)
            params.addElement(new Param(XMLP_CLASSNAME,Param.VALUESET,
                "<CLASSNAME NAME=\""+className+"\"/>"));
        else
            throw new CIMException(CIMException.CIM_ERR_INVALID_CLASS);
        params.addElement(new Param(XMLP_LOCAL,localOnly));
        params.addElement(new Param(XMLP_QUAL, includeQualifiers));
        params.addElement(new Param(XMLP_ORIGIN, includeClassOrigin));

        if (propertyList != null) 
        {
            extra="<IPARAMVALUE NAME=\""+XMLP_PROPERTYLIST+"\"><VALUE.ARRAY>";
            for(int i=0;i<propertyList.length;i++)
            extra=extra.concat("<VALUE>"+propertyList[i]+"</VALUE>");
            extra=extra.concat("</VALUE.ARRAY></IPARAMVALUE>");         
        }

        //result= intrinsicMethod(output, path, commandName, params);
        result= intrinsicMethod(output, path, commandName, params, extra);
        
        if (result==null) 
        {
            return(null);
        }
        // If there is a VALUE.NAMEDOBJECT node, then assume the class
        // is embedded within it - needed because some implementations
        // are still using this in the returns.
        if (result.getToken()==XMLNode.XML_ELEMENT_VALUE_NAMEDOBJECT) 
            result=result.getChild();
        else if (result.getToken()==XMLNode.XML_ELEMENT_VALUE_OBJECT)
            result=result.getChild();

        // Now find XML_ELEMENT_CLASS element
        if (result!=null)
            result=result.findElement(XMLNode.XML_ELEMENT_CLASS);
        if (result==null) 
        {
            //
            // Some kind of failure occurred but no exception has been
            // thrown yet...so throw one!
            //
            throw new CIMException(CIMException.CIM_ERR_FAILED);
        }

        return( new CIMClass(result) );
    }    
    
}
